#!/bin/bash
#This script will help in setup kerberos 
#This script can be used for both ui as well as command line

KEYTAB="/usr/local/safesquid/security/HTTP.keytab"
TMP_FILE="/tmp/safesquid/fetchinfo"

DEPENDENCY_CHECK(){

case "$1" in 
			--package)
				PACKAGE=`dpkg -l | grep $2`
				echo "checking $2 package"
				[ "x" != "x$PACKAGE" ] || aptitude -q install $2 -y 
				[ "x" != "x$PACKAGE" ] && return; 				
			 ;;
			 --command)
				SEARCH=`which $2`
				[ "x" != "x$SEARCH" ] || aptitude -q install $2 -y 
				[ "x" != "x$SEARCH" ] && return; 				
			 ;;
			 *)
				    echo "bad argument"
			 ;;
esac

}

REMOVE_FILES(){
	[ -f $1 ] || return;
	[ -f $1 ] && rm -f $1
}

CONNECTIVITY_TEST(){
	CONT=2
	count=$(ping -c ${CONT} $1 | grep 'received' | awk -F',' '{ print $2 }' | awk '{ print $1 }')
}


XML_BUILDER(){
	
	declare -n xmldata=$1
	
	echo '<?xml version="1.0" ?>'
	echo "<kerberos>"
	
	for i in "${!xmldata[@]}"; do 
	echo "<$i>${xmldata[$i]}</$i>"
	done	
	echo "</kerberos>\n"
}

FETCH_INFO(){
 
 declare -A AD_DETAIL
 
 CONNECTIVITY_TEST $1
 
 > ${TMP_FILE}
 
 if [ "x$count" == "x0" ]; then    	    
	AD_DETAIL[error]="Server unreachable"
 else
	curl -vv ldap://$1 > ${TMP_FILE} 2>&1
	
	AD_DETAIL[basedn]=$(cat ${TMP_FILE} | grep rootDomainNamingContext | awk '{print $2}')
	AD_DETAIL[fqdn]=$(cat ${TMP_FILE} | grep dnsHostName | awk '{print $2}')
	AD_DETAIL[domain]=$(cat ${TMP_FILE} | grep ldapServiceName | awk '{print $2}' | cut -d: -f1)
	
	[ "x" == "${AD_DETAIL[fqdn]}x" -o "x" == "${AD_DETAIL[basedn]}x" ] && AD_DETAIL[error]="Not an AD server" 
	fi
 
 XML_BUILDER AD_DETAIL
 
 unset AD_DETAIL
 
}

NTPDATE_SYNC(){
	sudo sh -c "ntpdate ${AD_FQDN}"
}

BIND_SETUP(){
	

sudo sh -c "cat << _EOF > /etc/bind/named.conf.local
zone "${AD_DOMAIN}" {
        type stub;
        masters {${AD_IP_ADDRESS};};
		};

_EOF"


DNS_ENTRY=$( cat /etc/bind/named.conf.options | grep -w '//dnssec-validation auto;' | wc -l )
LISTEN_V6=$( cat /etc/bind/named.conf.options | grep -w '//listen-on-v6 { any; }' | wc -l )
LISTEN_V4=$( cat /etc/bind/named.conf.options | grep -w 'listen-on {127.0.0.1;}' | wc -l )
RESOLVE=$( cat /etc/resolv.conf | grep -w '#search' | wc -l )

if [ "x${DNS_ENTRY}" != "x1" ];then	
	sudo sh -c "sed -i 's/dnssec-validation/\/\/dnssec-validation/g' /etc/bind/named.conf.options"
fi

if [ "x${LISTEN_V6}" != "x1" ];then
	sudo sh -c "sed -i 's/listen-on-v6/\/\/listen-on-v6/g' /etc/bind/named.conf.options"
fi

if [ "x${RESOLVE}" != "x1" ];then
	sudo sh -c "sed -i 's/search /#search/g' /etc/resolv.conf"
fi

if [ "x${LISTEN_V4}" == "x0" ];then
sudo sh -c "sed -i '22i\
 dnssec-validation no; \n listen-on {127.0.0.1;};
' /etc/bind/named.conf.options"
fi

sudo sh -c "unlink /etc/resolv.conf"
sudo sh -c "ln -fs /run/resolvconf/resolv.conf /etc/resolv.conf"
sudo sh -c "/etc/init.d/bind9 restart"

}

KRB_CONFIGURATION(){
sudo sh -c "cp -v /etc/krb5.conf /etc/krb5.conf.default`date +%d%m%Y:%H:%M:%S`"

sudo sh -c "cat << _EOF > /etc/krb5.conf 
[logging]
default = FILE:/var/log/krb5libs.log
kdc = FILE:/var/log/krb5kdc.log
admin_server = FILE:/var/log/kadmind.log

[libdefaults]
default_realm = ${AD_DOMAIN^^}
dns_lookup_realm = false 
dns_lookup_kdc = false 
ticket_lifetime = 720h 
renew_lifetime = 90d 
forwardable = true 
default_keytab_name = ${KEYTAB}

; for Windows 2003
default_tgs_enctypes = rc4-hmac des-cbc-crc des-cbc-md5
default_tkt_enctypes = rc4-hmac des-cbc-crc des-cbc-md5
permitted_enctypes = rc4-hmac des-cbc-crc des-cbc-md5

; for Windows 2008 with AES
default_tgs_enctypes = aes256-cts-hmac-sha1-96 rc4-hmac des-cbc-crc des-cbc-md5
default_tkt_enctypes = aes256-cts-hmac-sha1-96 rc4-hmac des-cbc-crc des-cbc-md5
permitted_enctypes = aes256-cts-hmac-sha1-96 rc4-hmac des-cbc-crc des-cbc-md5 

[realms]

${AD_DOMAIN^^} = {
kdc = ${AD_FQDN}:88
admin_server = ${AD_FQDN}:749
default_domain = ${AD_DOMAIN}
}

[domain_realm]
.${AD_DOMAIN} = ${AD_DOMAIN^^}
${AD_DOMAIN} = ${AD_DOMAIN^^}

_EOF"

}

BUILD_MSKTUTIL(){
	
	declare -A AD_DETAIL

	sudo sh -c "echo '${AD_ADMIN_PASSWD}' | /usr/bin/kinit ${AD_ADMIN_USER}" > /dev/null 2>&1

	if [ $? -eq 0 ]; then
		AD_DETAIL[PASSWD_MSG]="Correct password"
		sudo sh -c "/usr/local/bin/msktutil -c -b 'CN=COMPUTERS' -s HTTP/${BASE_FQDN} -k ${KEYTAB} --computer-name ${BASE_HOSTNAME^^}-K --upn HTTP/${BASE_FQDN} --server ${AD_FQDN}" > /dev/null 2>&1				
	else
		AD_DETAIL[PASSWD_MSG]="Wrong Password"
	fi
	
	if [ -f ${KEYTAB} ];then	
		sudo sh -c "chmod 440 ${KEYTAB}"
		sudo sh -c "chown ssquid:root ${KEYTAB}"
		sudo sh -c "export KRB5_KTNAME=${KEYTAB}"
		
		AD_DETAIL[MSG]="Successfully generated keytab"
	else
		AD_DETAIL[MSG]="Could not generated keytab"
	fi
	
	XML_BUILDER AD_DETAIL
	
}

RESOLVE_AND_DOMAIN_CHECK(){
	
	if [ "x${AD_DOMAIN}" == "x${BASE_DOMAIN}" ];then	
		if [ "x${BASE_HOSTNAME}" == "x${BASE_FQDN}" ];then										
			sudo sh -c "sed -i 's/${BASE_HOSTNAME}/${BASE_HOSTNAME}.${AD_DOMAIN} ${BASE_HOSTNAME}/g' /etc/hosts"			
		fi
	else				
		sudo sh -c "sed -i 's/${BASE_FQDN}/${BASE_HOSTNAME}.${AD_DOMAIN}/g' /etc/hosts"
	fi
	
	BASE_FQDN="$(hostname -f)"
	
	BASE_HOSTENTRY=$(cat /etc/hosts | grep ${BASE_FQDN})
	AD_HOSTENTRY=$(cat /etc/hosts | grep ${AD_FQDN})
	
	[ "x${BASE_HOSTENTRY}" == "x" ] && sudo sh -c "echo '${BASE_IP_ADDRESS} ${BASE_HOSTNAME}.${AD_DOMAIN} ${BASE_HOSTNAME}' >> /etc/hosts" 
	[ "x${AD_HOSTENTRY}" == "x" ] && sudo sh -c "echo '${AD_IP_ADDRESS} ${AD_FQDN}' >> /etc/hosts"
		
}

INSTALL_DEPENDENCY_PACKAGE(){

	SUDOERS=$( cat /etc/sudoers | grep -w 'ssquid  ALL=(ALL) NOPASSWD:ALL' | wc -l )

	if [ "x${SUDOERS}" != "x1" ];then
		echo "ssquid  ALL=(ALL) NOPASSWD:ALL" >> /etc/sudoers
	fi
	
	if [ ! -f "/usr/local/bin/msktutil" ];then		
		sudo sh -c "wget 'http://downloads.safesquid.net/appliance/source/msktutil' -O /usr/local/bin/msktutil"
		sudo sh -c "chmod 755 /usr/local/bin/msktutil"
	fi
	
	export DEBIAN_FRONTEND=noninteractive
	
	DEPENDENCY_CHECK --package heimdal-clients
	DEPENDENCY_CHECK --package libsasl2-modules
	DEPENDENCY_CHECK --package libkrb5-3
	DEPENDENCY_CHECK --package ldap-utils		
	DEPENDENCY_CHECK --package libsasl2-modules-gssapi-heimdal
	DEPENDENCY_CHECK --package zlib1g
	DEPENDENCY_CHECK --package libwind0-heimdal
	DEPENDENCY_CHECK --package libudns0
	DEPENDENCY_CHECK --package libtasn1-6
	DEPENDENCY_CHECK --package libstdc++6
	DEPENDENCY_CHECK --package libsqlite3-0
	DEPENDENCY_CHECK --package libsasl2-2
	DEPENDENCY_CHECK --package libroken18-heimdal
	DEPENDENCY_CHECK --package libp11-kit0
	DEPENDENCY_CHECK --package libkrb5-26-heimdal
	DEPENDENCY_CHECK --package libhx509-5-heimdal
	DEPENDENCY_CHECK --package libheimntlm0-heimdal
	DEPENDENCY_CHECK --package libheimbase1-heimdal
	DEPENDENCY_CHECK --package libhcrypto4-heimdal
	DEPENDENCY_CHECK --package libgssapi3-heimdal
	DEPENDENCY_CHECK --package libgpg-error0
	DEPENDENCY_CHECK --package libgnutls26
	DEPENDENCY_CHECK --package libgcrypt11
	DEPENDENCY_CHECK --package libgcc1
	DEPENDENCY_CHECK --package libffi6
	DEPENDENCY_CHECK --package libcomerr2
	DEPENDENCY_CHECK --package libasn1-8-heimdal	
	DEPENDENCY_CHECK --package curl	
	DEPENDENCY_CHECK --package bind9
	
	export DEBIAN_FRONTEND=
		
}

VALIDATE_ENVIRONMENT(){
	declare -A AD_DETAIL
	
	[ -f ${KEYTAB} ] && AD_DETAIL[keytab]="found" || AD_DETAIL[keytab]="not found"
	
	AD_DETAIL[adfqdn]=$(cat /etc/krb5.conf | grep -w 'kdc =' | grep 88 | cut -d'=' -f2 | cut -d':' -f1)
	AD_DETAIL[domain]=$(cat /etc/krb5.conf | grep -w 'default_domain' | cut -d'=' -f2 | cut -d':' -f1)
	
	XML_BUILDER AD_DETAIL
}


MAIN(){

	declare -A VALUE
	
	ARG=$#	
	ARG_INPUT=$@
	n=1
	
	
	#first argument
	FIRST_ARG=$1
	
	#Machine on which kerberos is to setup
	BASE_IP_ADDRESS="$(ip route get 1 | awk '{print $NF;exit}')"
	BASE_FQDN="$(hostname -f)"
	BASE_HOSTNAME="$(hostname)"
	BASE_DOMAIN="$(dnsdomainname)"

	#Active directory details
	AD_IP_ADDRESS=""	
	AD_FQDN=""
	AD_DOMAIN=""	
	
	#credentials to run mskutils	
	AD_ADMIN_USER=""
	AD_ADMIN_PASSWD=""
		
	#total argument passed	
	ARG_TOTAL=$( expr ${ARG} / 2 )
	
	#package check
	if [ "x${FIRST_ARG}" == "xpackage" ];then
	  INSTALL_DEPENDENCY_PACKAGE
	  exit 0
	fi
	
	#package check
	if [ "x${FIRST_ARG}" == "xvalidate" ];then
	  VALIDATE_ENVIRONMENT
	  exit 0
	fi
	
	
	#argument to array
	IFS=' ' read -r -a array <<< "${ARG_INPUT}"

	while [ ${ARG_TOTAL} -ne 0 ]
	do					
		content=${array[$( expr $n - 1 )]}
		VALUE["${content}"]="${array[$n]}"
		
		n=$(expr $n + 2)							
		ARG_TOTAL=$(expr ${ARG_TOTAL} - 1)
	done		
	
	#fetch AD information
	[[ ! -z ${VALUE[ip]} ]] && AD_IP_ADDRESS="${VALUE[ip]}"
	[[ ! -z ${VALUE[domain]} ]] && AD_DOMAIN="${VALUE[domain]}"
	[[ ! -z ${VALUE[fqdn]} ]] && AD_FQDN="${VALUE[fqdn]}"	
	
	[[ ! -z ${VALUE[username]} ]] && AD_ADMIN_USER="${VALUE[username]}"
	[[ ! -z ${VALUE[password]} ]] && AD_ADMIN_PASSWD="${VALUE[password]}"
	
	if [[ ! -z ${AD_IP_ADDRESS}  && "${AD_DOMAIN}x" == "x" ]]
	then
		[ -f ${KEYTAB} ] && sudo sh -c "mv ${KEYTAB} ${KEYTAB}_bkp" 
		FETCH_INFO ${AD_IP_ADDRESS}
	fi
	
	if [[ ! -z ${AD_DOMAIN} ]];then				
		RESOLVE_AND_DOMAIN_CHECK 
		BIND_SETUP
		NTPDATE_SYNC	
		KRB_CONFIGURATION
	fi
	
	if [[ ! -z ${AD_ADMIN_USER} ]];then		
		BUILD_MSKTUTIL		
		sudo sh -c "sed -i 's/HOSTNAME\=.*/HOSTNAME\=${BASE_FQDN}/g'  /opt/safesquid/default/startup.ini"
		sudo sh -c "sed -i 's/DOMAIN\=.*/DOMAIN\=${BASE_DOMAIN}/g' /opt/safesquid/default/startup.ini"		
	fi
}

MAIN $*
exit 0
